unit UnitMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, VCLTee.TeEngine,
  VCLTee.Series, VCLTee.TeeProcs, VCLTee.Chart, Vcl.StdCtrls, Vcl.ExtCtrls,
  UnitCameraDefs;

type
  TCCDThread = class(TThread)
  private
    FwCCDBuf : array of Word;
    FCCDRead : DWORD;
    procedure UpDateChart;
  protected
    procedure Execute; override;
  public
    FActive  : Boolean;
    FTimeOut : DWORD;
  end;

  TfmMain = class(TForm)
    Panel1: TPanel;
    btn_enum: TButton;
    cb_devices: TComboBox;
    btn_opendevice: TButton;
    btn_info: TButton;
    btn_start: TButton;
    Panel2: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lb_gain: TLabel;
    lb_offset: TLabel;
    Label5: TLabel;
    Label4: TLabel;
    cb_mode: TComboBox;
    ed_exptime: TEdit;
    btn_exptime: TButton;
    tb_gain: TTrackBar;
    tb_offset: TTrackBar;
    cb_fullscale: TComboBox;
    Chart1: TChart;
    Series1: TLineSeries;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure btn_enumClick(Sender: TObject);
    procedure cb_devicesChange(Sender: TObject);
    procedure btn_opendeviceClick(Sender: TObject);
    procedure btn_startClick(Sender: TObject);
    procedure cb_modeChange(Sender: TObject);
    procedure btn_exptimeClick(Sender: TObject);
    procedure tb_gainChange(Sender: TObject);
    procedure tb_offsetChange(Sender: TObject);
    procedure cb_fullscaleChange(Sender: TObject);
    procedure btn_infoClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure to_statusbar(stxt : string);
    procedure closeusbdevice;
    procedure StartCCDThread;
    procedure TerminateCCDThread;
    procedure WaitForCCDThread;
    procedure SyncParam;
  end;

var
  fmMain: TfmMain;
  dwPrivateMsg    : DWORD;
  CCDThreadReader : TCCDThread;

implementation

{$R *.dfm}

uses UnitInfo;

{ TCCDThread }
procedure TCCDThread.Execute;
var dwErr : DWORD;
begin
  FActive := true;
  while not Terminated do
  begin
    if ls_getpipe(@FwCCDBuf[0],PACKET_LENGTH,FCCDRead) = 0 then
    begin
        Synchronize(UpdateChart);
    end else
    begin
      dwErr := ls_waitforpipe(FTimeOut);
      if dwErr = ERROR_INVALID_HANDLE then break
    end;
  end;
  FActive := false;
end;

procedure TCCDThread.UpDateChart;
var i : word;
begin
  for i := 0 to (FCCDRead div 2)-1 do
  begin
    fmMain.Series1.YValues.Value[i] := FwCCDBuf[i];
  end;
  fmMain.Series1.YValues.Modified := true;
  fmMain.Series1.Repaint;

  if fmMain.cb_mode.ItemIndex = 0 then
  begin
    fmMain.btn_start.Tag := 0;
    fmMain.btn_start.Caption := 'Start';
  end;
end;

procedure TfmMain.to_statusbar(stxt: string);
begin
  StatusBar1.Panels[0].Text := stxt;
end;

procedure TfmMain.FormCreate(Sender: TObject);
var i : word;
begin
  Series1.Clear;
  Chart1.BottomAxis.SetMinMax(0,NUMBER_OF_PIXELS-1);
  for i := 0 to 2047 do
    Series1.AddXY(i,0);

  cb_mode.ItemIndex := 2;
  cb_fullscale.ItemIndex := 0;

  dwPrivateMsg := ls_initialize(_32MB,PACKET_LENGTH,NORMAL_PRIORITY_CLASS, THREAD_PRIORITY_ABOVE_NORMAL, 'CX_USB_LINE_CAMERA');
  if dwPrivateMsg <> 0 then to_statusbar('APPLICATION INITIALIZED');
end;

procedure TfmMain.btn_enumClick(Sender: TObject);
var Cnt, i : ShortInt;
    dwErr : DWORD;
begin
  cb_devices.Clear;

  if ls_currentdeviceindex > -1 then
    closeusbdevice;

  Cnt := ls_enumdevices;

  to_statusbar(IntToStr(Cnt)+' device(s) found');
  if Cnt > 0 then
  begin
    for i := 0 to Cnt - 1 do
      cb_devices.Items.Add(String(ls_getproductname(i))+' '+String(ls_getserialnumber(i)));

    dwErr := ls_opendevicebyindex(0);
    if  dwErr = RET_OK then
    begin
      SyncParam;
      StartCCDThread;
      cb_devices.ItemIndex := 0;
      btn_opendevice.Caption := 'Close Device';
    end;
    to_statusbar(String(ls_geterrorstring(dwErr)));
  end else to_statusbar('NO USB DEVICES ATTACHED!');
end;

procedure TfmMain.btn_exptimeClick(Sender: TObject);
var dwErr : DWORD;
    dwexptime : DWORD;
begin
  dwexptime := StrToIntDef(ed_exptime.Text,10000);
  ed_exptime.Text := IntToStr(dwexptime);
  dwErr := ls_setinttime(dwexptime,10);
  to_statusbar('SET INT TIME: '+ls_geterrorstring(dwerr));
end;

procedure TfmMain.btn_infoClick(Sender: TObject);
begin
  fmInfo.ShowModal;
end;

procedure TfmMain.btn_opendeviceClick(Sender: TObject);
var dwErr : DWORD;
begin
  if cb_devices.Items.Count = 0 then exit;

  if ls_currentdeviceindex > -1 then
  begin
    closeusbdevice;
    btn_opendevice.Caption := 'Open Device';
    to_statusbar('USB DEVICE CLOSED');
  end else
  begin
    dwErr := ls_opendevicebyindex(cb_devices.ItemIndex);
    if  dwErr = RET_OK then
    begin
      SyncParam;
      StartCCDThread;
      btn_opendevice.Caption := 'Close Device';
    end;
    to_statusbar(String(ls_geterrorstring(dwErr)));
  end;
end;

procedure TfmMain.btn_startClick(Sender: TObject);
var dwErr : DWORD;
begin
  btn_start.Tag := (not btn_start.Tag) and $01;
  dwErr := ls_setstate(btn_start.Tag,10);
  if dwErr = RET_OK then
  begin
    if btn_Start.Tag = $01 then btn_Start.Caption := 'Stop'
    else btn_Start.Caption := 'Start';
  end else btn_start.Tag := (not btn_start.Tag) and $01;
  to_statusbar('SET STATE: '+ls_geterrorstring(dwerr));
end;

procedure TfmMain.cb_devicesChange(Sender: TObject);
var dwErr : DWORD;
begin
  if ls_currentdeviceindex > -1 then
  begin
    closeusbdevice;
    btn_opendevice.Caption := 'Open Device';
    to_statusbar('USB DEVICE CLOSED');
  end else
  begin
    dwErr := ls_opendevicebyindex(cb_devices.ItemIndex);
    if  dwErr = RET_OK then
    begin
      SyncParam;
      StartCCDThread;
      btn_opendevice.Caption := 'Close Device';
    end;
    to_statusbar(String(ls_geterrorstring(dwErr)));
  end;
end;

procedure TfmMain.cb_fullscaleChange(Sender: TObject);
var dwerr : dword;
begin
  dwerr := ls_setadcconfig(cb_fullscale.ItemIndex shl 7);
  to_statusbar('SET FULL SCALE: '+ls_geterrorstring(dwerr));
end;

procedure TfmMain.cb_modeChange(Sender: TObject);
var dwerr : DWORD;
begin
  dwerr := ls_setmode(cb_mode.ItemIndex,10);
  if dwerr = RET_OK then
  begin
    if cb_mode.ItemIndex = 0 then
    begin
      btn_start.Tag := 0;
      btn_start.Caption := 'Start';
    end;
  end;
  to_statusbar('SET MODE: '+ls_geterrorstring(dwerr));
end;

procedure TfmMain.closeusbdevice;
begin
  TerminateCCDThread;
  ls_closedevice;
  WaitForCCDThread;
end;

procedure TfmMain.StartCCDThread;
begin
  CCDThreadReader := TCCDThread.Create(True);
  CCDThreadReader.Priority := tpHighest;
  CCDThreadReader.FreeOnTerminate := false;
  CCDThreadReader.FActive := false;
  CCDThreadReader.FTimeOut := _1S;
  SetLength(CCDThreadReader.FwCCDBuf,NUMBER_OF_PIXELS);
  CCDThreadReader.Start;
end;

procedure TfmMain.TerminateCCDThread;
begin
  if (CCDThreadReader <> nil) and (CCDThreadReader.FActive) then
    CCDThreadReader.Terminate;
end;

procedure TfmMain.WaitForCCDThread;
begin
  if (CCDThreadReader <> nil) then
  begin
    CCDThreadReader.WaitFor;
    SetLength(CCDThreadReader.FwCCDBuf,0);
    CCDThreadReader := nil;
  end;
end;

procedure TfmMain.SyncParam;
Var dwErr : DWORD;
    wValue : WORD;
    fValue : Single;
    iValue : Integer;
    dwValue : DWORD;
    ucValue : Byte;
begin
  dwErr := ls_getadcconfig(wValue);
  if dwErr = RET_OK then
  begin
    cb_fullscale.OnChange := nil;
    cb_fullscale.ItemIndex := (wvalue shr 7) and $01;
    cb_fullscale.OnChange := cb_fullscaleChange;
  end;

  dwErr := ls_getadcpga1(wValue);
  if dwErr = RET_OK then
  begin
    tb_gain.OnChange := nil;
    fValue := 6 / (1+(5*((63-wValue)/63)));
    if fValue < 1 then fValue := 1;
    if fValue > 6 then fValue := 6;
    tb_gain.Position  := wValue;
    lb_gain.caption := Format('%.4f V/V',[fValue]);
    tb_gain.OnChange := tb_gainChange;
  end;

  dwErr := ls_getadcoffset1(wValue);
  if dwErr = RET_OK then
  begin
    tb_offset.OnChange := nil;

    ivalue := wvalue and $FF;
    if ((wvalue and $100) = $100) then ivalue := ivalue * (-1);

    fvalue := (ivalue * 1.2);
    if fvalue < -300 then fvalue := -300;
    if fvalue > 300 then fvalue := 300;
    tb_offset.position := ivalue;
    lb_offset.caption := Format('%.1f mV',[fValue]);
    tb_offset.OnChange := tb_offsetChange;
  end;

  dwErr := ls_getinttime(dwvalue,10);
  if dwErr = RET_OK then
    ed_exptime.Text := IntToStr(dwvalue);

  dwErr := ls_getstate(ucValue,10);
  if dwErr = RET_OK then
  begin
    btn_start.Tag := ucValue;
    if btn_start.tag = 0 then btn_start.caption := 'Start'
    else btn_start.caption := 'Stop';
  end;

  dwErr := ls_getmode(ucValue,10);
  if dwErr = RET_OK then
  begin
    cb_mode.OnChange := nil;
    cb_mode.ItemIndex := ucValue and $03;
    cb_mode.OnChange := cb_modeChange;
  end;
end;

procedure TfmMain.tb_gainChange(Sender: TObject);
var dwerr : dword;
begin
  dwerr := ls_setadcpga1(tb_gain.Position);
  lb_gain.caption := Format('%.4f V/V',[6 / (1+(5*((63-tb_gain.Position)/63)))]);
  to_statusbar('SET GAIN: '+ls_geterrorstring(dwerr));
end;

procedure TfmMain.tb_offsetChange(Sender: TObject);
var fNewValue : Single;
    wValue : word;
    Sign   : word;
    dwerr  : dword;
begin
  fNewValue := tb_offset.Position * 1.2;
  if (fNewValue < -300) then fNewValue := -300;
  if (fNewValue > 300) then fNewValue := 300;

  wvalue := abs(tb_offset.Position);
  if tb_offset.Position < 0 then sign := $100
  else sign := 0;
  wvalue := wvalue or sign;
  dwerr := ls_setadcoffset1(wvalue);

  lb_offset.caption := Format('%.1f mV',[fNewValue]);
  to_statusbar('SET OFFSET: '+ls_geterrorstring(dwerr));
end;

end.
